module vgafla_g(clock, vga_hsync , vga_vsync, vga_red0, vga_green0, vga_blue0, 
            	vga_red1, vga_green1, vga_blue1);

	input 	clock;
	output 	vga_hsync, vga_vsync; // sync signals for monitor
	output 	vga_red0, vga_green0, vga_blue0, vga_red1, vga_green1, vga_blue1;

	wire 	[9:0] YPos;	// [0..479]     
	wire 	[10:0] XPos;	// [0...1287]
	wire 	Valid;
	wire    [7:0] tcgrom_d, tcgrom_q;
	wire    [5:0] char_selection_d, char_selection_q;
	reg 	color;
	
	wire 	[39:0] slow_clk_d, slow_clk_q;	
	wire 	vga_hsync_d, vga_hsync_q;
	wire 	vga_vsync_d, vga_vsync_q; 
	wire    [1:0] vga_red_d, vga_red_q;
	wire    [1:0] vga_green_d, vga_green_q;
	wire 	[1:0] vga_blue_d, vga_blue_q;

	reg    [5:0] char_selection;
	
///////////////////////////////////////////

	// Display the correct character
	always @(YPos or XPos or slow_clk_q) 
	  begin
	    if (YPos[9:5] == 4'b0)
	      case (XPos[10:5]) 
	        6'd0 : char_selection = 6'o30;
	        6'd1 : char_selection = {2'b11, slow_clk_q[35:32]};
	        6'd2 : char_selection = {2'b11, slow_clk_q[31:28]};
	        6'd3 : char_selection = {2'b11, slow_clk_q[27:24]};
	        6'd5 : char_selection = 6'o31;
	        6'd6 : char_selection = {2'b11, ~slow_clk_q[35:32]};
	        6'd7 : char_selection = {2'b11, ~slow_clk_q[31:28]};
	        6'd8 : char_selection = {2'b11, ~slow_clk_q[27:24]};
	        6'd10 : char_selection = 6'o14;
	        6'd11 : char_selection = {5'b11000, 1'b0};
	        6'd13 : char_selection = 6'o22;
	        6'd14 : char_selection = {5'b11000, 1'b1};
                default :  char_selection = 6'b100000;
              endcase 
	    else 
               char_selection = {YPos[5], XPos[9:5]};
	  end

	// Register the output of the character generator
	assign char_selection_d = char_selection;
	
	// Register the output of the tcgrom
        tcgrom tcgrom(.addr({char_selection_q, YPos[4:2]}), .data(tcgrom_d));

        always @(XPos or tcgrom_q)
          begin  	
	    case (XPos[4:2])
	      3'h0 : color = tcgrom_q[7];
	      3'h1 : color = tcgrom_q[6];
	      3'h2 : color = tcgrom_q[5];
	      3'h3 : color = tcgrom_q[4];
	      3'h4 : color = tcgrom_q[3];
	      3'h5 : color = tcgrom_q[2];
	      3'h6 : color = tcgrom_q[1];
	      3'h7 : color = tcgrom_q[0];	 	      	 	      
	    endcase 
	  end  
	  
///////////////////////////////////////////

	assign slow_clk_d = slow_clk_q + 32'b1;

	sync_gen50 syncVGA( .clk(clock), .CounterX(XPos), .CounterY(YPos), .Valid(Valid),
	                        .vga_h_sync(vga_hsync_d), .vga_v_sync(vga_vsync_d));

///////////////////////////////////////////
// generates the RGB signals based on raster position
	
	wire red0 = Valid &&  	((XPos <200 ) ||
				 (YPos > 80 ) );
	wire red1 = Valid && 	((XPos > 200) && (XPos  < 400) ||
			      	 (YPos > 80 ) && (YPos  < 160) ); 			
	wire green0 = Valid && 	((XPos > 400) && (XPos  < 600) ||
				 (YPos > 160) && (YPos  < 240) );
	wire green1 =  Valid && ((XPos > 600) && (XPos  < 800) ||
				 (YPos > 240) && (YPos  < 320) );
	wire blue0 = Valid && 	((XPos > 800) && (XPos  < 1000)||
				 (YPos > 320) && (YPos  < 400) );
	wire blue1 = Valid && 	((XPos > 1000)|| (YPos  > 400) );
		
	assign vga_red_d[0]   = red0 ? color /* 1'b1 */ :1'b0;
	
	assign vga_green_d[0] = green0 ? color /*1'b1 */ :1'b0;
	
	assign vga_blue_d[0]  = blue0  ? color /* 1'b1 */ :1'b0;
	
	assign vga_red_d[1]   = red1 ? color /* 1'b1 */ : 1'b0;
	
	assign vga_green_d[1] = green1 ?  color /* 1'b1 */: 1'b0;
	
	assign vga_blue_d[1]  = blue1?  color /* 1'b1 */ :1'b0 ;

   assign vga_vsync = vga_vsync_q;
   assign vga_hsync = vga_hsync_q;
   assign vga_red1 = vga_red_q[1];
   assign vga_red0 = vga_red_q[0];
   assign vga_green1 = vga_green_q[1];
   assign vga_green0 = vga_green_q[0];
   assign vga_blue1 = vga_blue_q[1];
   assign vga_blue0 = vga_blue_q[0];

   // Output flops
   dff #(8) tcgrom_reg (.clk(clock), .d(tcgrom_d), .q(tcgrom_q));
   dff #(6) char_selection_reg (.clk(clock), .d(char_selection_d), 
   	.q(char_selection_q));
   dff vga_vsync_reg (.clk(clock), .d(vga_vsync_d), .q(vga_vsync_q));
   dff vga_hsync_reg (.clk(clock), .d(vga_hsync_d), .q(vga_hsync_q));
   dff #(2) vga_red_reg (.clk(clock), .d(vga_red_d), .q(vga_red_q));
   dff #(2) vga_green_reg (.clk(clock), .d(vga_green_d), .q(vga_green_q));
   dff #(2) vga_blue_reg (.clk(clock), .d(vga_blue_d), .q(vga_blue_q));  
   dffre #(40) slow_clk_reg (.clk(clock), .en(1'b1), .r(1'b0), .d(slow_clk_d), .q(slow_clk_q));


endmodule


